def create_box(output: str, margen_length=2):
    """
    Creates a box around the given output string with a specified margin length.

    Args:
        output (str): The string to be boxed.
        margen_length (int): The length of the margin. Defaults to 2.

    Returns:
        str: The boxed string.
    """
    margen = " " * margen_length
    box_length = max(output.split("\n"), key=len)

    boxed_output = "\n".join(
        [
            f"{margen}{line}{' ' * (len(box_length) - len(line))}{margen}"
            for line in output.split("\n")
        ]
    )
    return f"\n{boxed_output}\n"



import ctypes
import platform


if platform.system() == "Windows":
    kernel32 = ctypes.windll.kernel32
    kernel32.SetConsoleMode(kernel32.GetStdHandle(-11), 7)

COMMANDS = {
    # Labels
    "info": (33, "[!] "),
    "que": (34, "[?] "),
    "bad": (31, "[-] "),
    "good": (32, "[+] "),
    "run": (97, "[~] "),
    # Colors
    "green": 32,
    "leen": 92,
    "lightgreen": 92,
    "grey": 37,
    "black": 30,
    "red": 31,
    "lred": 91,
    "lightred": 91,
    "cyan": 36,
    "lcyan": 96,
    "lightcyan": 96,
    "blue": 34,
    "lblue": 94,
    "lightblue": 94,
    "purple": 35,
    "yellow": 93,
    "white": 97,
    "laurel": 95,
    "lightpurple": 95,
    "orange": 33,
    # Styles
    "bg": ";7",
    "bold": ";1",
    "italic": "3",
    "under": "4",
    "strike": "09",
}


def _gen(string, prefix, key):
    colored = prefix if prefix else string
    not_colored = string if prefix else ""
    result = "\033[{}m{}\033[0m{}".format(key, colored, not_colored)
    return result


for key, val in COMMANDS.items():
    value = val[0] if isinstance(val, tuple) else val
    prefix = val[1] if isinstance(val, tuple) else ""
    locals()[key] = lambda s, prefix=prefix, key=value: _gen(s, prefix, key)



from dataclasses import dataclass

import black

from .hue import bg, red, green, orange
from .utils import create_box

from .types import BoxTypes


@dataclass
class Noodle:
    def print(self, object, line_length=50):
        """
        Formats and print the given object as a string using Black.

        Args:
            object: The object to be printed.
            line_length (int): The maximum length of each line in the output.

        Returns:
            None
        """
        print(self._output(object, line_length=line_length))

    def block(self, object, line_length=50, margen_length=2, type: BoxTypes = "info"):
        """
        Formats the given object with Black as a block of text with a colored
        background.

        Args:
            object: The object to format.
            line_length (int): The maximum length of each line in the block.
            margen_length (int): The number of spaces to include in the left and
            right margins of the block.

        Returns:
            None
        """
        output = f"\n{self._output(object, line_length=line_length)}"
        final_output = create_box(output, margen_length)
        match type:
            case "info":
                print(bg(final_output))
            case "error":
                print(bg(red(final_output)))
            case "success":
                print(bg(green(final_output)))
            case "warning":
                print(bg(orange(final_output)))
            case _:
                print(final_output)

    def _output(self, object, line_length=50):
        """
        Formats the given object as a string using Black.

        Args:
            object: The object to format as a string.
            line_length: The maximum line length for the formatted string.

        Returns:
            If the object is already a string, returns the object unchanged.
            Otherwise, returns the formatted string representation of the object.
        """
        if isinstance(object, str):
            return object

        return black.format_str(
            str(object),
            mode=black.FileMode(line_length=line_length),
        )
